#ifndef LOGGING_H_INCLUDED
#define LOGGING_H_INCLUDED

//XXXTODO

/****************************
*  ANSI C/C++ DEPENDENCIES  *
****************************/
#include <time.h>             // for time-related functions
#include <stdlib.h>           // for exit()
#include <stdio.h>            // for printf()
#include <arpa/inet.h>        // for uint16_t, etc.


/****************************
*  PRE-COMPILER DIRECTIVES  *
****************************/
using namespace std;
//#define LOGLEVEL_NOTE   0     // run of the mill info/debugging log entries
//#define LOGLEVEL_WARN   1     // entries about minor issues
//#define LOGLEVEL_ERROR  2     // entries about significant but recoverable issues
//#define LOGLEVEL_ABEND  3     // entries about non-recoverable errors (terminal)
enum LOGLEVEL{ LOGLEVEL_NOTE, LOGLEVEL_WARN, LOGLEVEL_ERROR, LOGLEVEL_ABEND };


/***************
*  PROTOTYPES  *
***************/
void LOG(const char*, LOGLEVEL=LOGLEVEL_NOTE);               // responsible for timestamped log messages
void LOG(const char*, char*, LOGLEVEL=LOGLEVEL_NOTE);        // same idea, but throw in a %s ala sprintf()
void LOG(const char*, uint16_t, LOGLEVEL=LOGLEVEL_NOTE);     // same idea, but throw in a %X ala sprintf()
void FormatIP4Addr(char*, uint32_t);


/**************************************
*  LOG()                              *
*  Simple/fast logging functionality  *
**************************************/
void LOG(const char* msg, LOGLEVEL level)
{
   const char* levelstr;
   switch(level)
   {
      case(LOGLEVEL_NOTE):
         levelstr = "NOTE";
         break;
      case(LOGLEVEL_WARN):
         levelstr = "WARN";
         break;
      case(LOGLEVEL_ERROR):
         levelstr = "ERROR";
         break;
      case(LOGLEVEL_ABEND):
         levelstr = "CRITICAL";
         break;
      default:
         levelstr = "UNKNOWN";
   } // end switch()

   // build a custom date/time stamp for our log entry
   char tstr[100];                  // static declaration of buf avoids heap
   time_t t = time(nullptr);        // declare a time_t instance with system time
   struct tm* p = localtime(&t);    // tm struct containing localized time
   strftime(tstr, 100, "%F %T", p); // make a pretty string

   // a little paranoia goes a long way...
   if(msg == nullptr)
      msg = "Invalid char* sent to LOG()";

   // now display the log entry to the console
   //printf("[%s][%s] %s\n", tstr, levelstr, msg);
   printf("[%s][%i][%s] %s\n", tstr, getpid(), levelstr, msg);

   // if this was a terminal error, we need to abend!
   if(level == LOGLEVEL_ABEND)
      exit(EXIT_FAILURE);

} // end LOG()


void LOG(const char* msg, char* str, LOGLEVEL level)
{
   // a little paranoia goes a long way...
   if(msg == nullptr)
      return;
   if(str == nullptr)
      LOG(msg, level);

   // okay, get on with it...
   char buf[255];
   sprintf(buf, msg, str);
   LOG(buf, level);

} // end LOG()


void LOG(const char* msg, const char* str, LOGLEVEL level)
{
   // a little paranoia goes a long way...
   if(msg == nullptr)
      return;
   if(str == nullptr)
      LOG(msg, level);

   // okay, get on with it...
   char buf[255];
   sprintf(buf, msg, str);
   LOG(buf, level);

} // end LOG()


void LOG(const char* msg, int num, LOGLEVEL level)
{
   // a little paranoia goes a long way...
   if(msg == nullptr)
      return;

   // okay, get on with it...
   char buf[255];
   sprintf(buf, msg, num);
   LOG(buf, level);

} // end LOG()


void LOG(const char* msg, uint16_t num, LOGLEVEL level)
{
   // a little paranoia goes a long way...
   if(msg == nullptr)
      return;

   // okay, get on with it...
   char buf[255];
   sprintf(buf, msg, num);
   LOG(buf, level);

} // end LOG()


// XXXTODOTODOTODO
void FormatIP4Addr(char* retval, uint32_t ipaddr)
{
   u_char* ip = (u_char*)&ipaddr;
   sprintf(retval, "%i.%i.%i.%i", ip[0], ip[1], ip[2], ip[3]);
} // end FormatIP4Addr()


#endif // LOGGING_H_INCLUDED
